@@ -1,5 +1,6 @@
-import os
 import tempfile
+import threading
+import os
 
 
 def get_class_name(model):
@@ -8,13 +9,18 @@ def get_class_name(model):
     return name
 
 
-def get_tmp_path():
+def get_temp_base():
+    tid = str(threading.get_ident())
     return os.path.join(tempfile._get_default_tempdir(),
+                        'onnc-bench', tid)
+
+
+def get_tmp_path():
+    temp_base = get_temp_base()
+
+    if not os.path.exists(temp_base):
+        os.makedirs(temp_base, exist_ok=True)
+    return os.path.join(temp_base,
                         next(tempfile._get_candidate_names()))
 
 
-def remove_if_exist(path):
-    if type(path) != str:
-        return
-    if os.path.exists(path):
-        os.remove(path)
