@@ -1,7 +1,9 @@
-from typing import Any, List
-
+from typing import Any, List, Union, Dict, Set
 from abc import abstractmethod
 from pathlib import Path
+
+from loguru import logger
+
 from onnc.bench.core.project import ProjectData
 from onnc.bench.core.deployment import Deployment
 
@@ -33,7 +35,7 @@ class IBuilder:
         pass
 
     @abstractmethod
-    def build(self, device) -> Any:
+    def build(self, target, converter_params: Dict = {}) -> Any:
         """build a project witch contains multiple models
 
         for model in model_ids:
@@ -42,5 +44,19 @@ class IBuilder:
         pass
 
     @abstractmethod
-    def save(self, output: Path) -> Deployment:
+    def save(self, output: Path) -> Union[Dict, Deployment]:
         pass
+
+    @property
+    def supported_devices(self) -> Dict:
+        pass
+
+    def get_device_id(self, target):
+        supported_devices = self.supported_devices
+        try:
+            return supported_devices[target]
+        except KeyError:
+            logger.error(f'`{target}` is not a supported device/format.')
+            logger.error(f'Supported devices/formats are: '
+                         f'{str([x for x in supported_devices])} ')
+            raise Exception(f'`{target}` is not supported')
