@@ -2,6 +2,7 @@ from typing import List
 from pathlib import Path
 from abc import abstractmethod
 import shutil
+from loguru import logger
 
 from .transformer import ModelTransformer
 from .model import Model
@@ -16,17 +17,13 @@ class SerializerRegistry(type):
 
     def __new__(cls, name, bases, attrs):
 
-        # inherent attrs in ModelTransformer but not overwrite them
-        for k in ModelTransformer.__dict__:
-            if k not in attrs:
-                attrs[k] = ModelTransformer.__dict__[k]
 
         new_cls = type.__new__(cls, name, bases, attrs)
         cls.REGISTRY.append(new_cls)
         return new_cls
 
 
-class Serializer(metaclass=SerializerRegistry):
+class Serializer(ModelTransformer, metaclass=SerializerRegistry):
 
     FORMAT = None
 
@@ -68,12 +65,37 @@ class FileSerializer(Serializer):
 
         if model.src.is_file():
             shutil.copy(model.src, dest)
+        else:
+            raise Exception(f'src [`{model.src}`] should be a path to a file.')
 
-        elif model.src.is_dir():
+        return Model(dest)
+
+    def serialize(self, model: Model, dest: Path) -> Model:
+        # return super().serialize(model, dest)
+        self.add_param('dest', dest)   # type: ignore[attr-defined]
+        m = self.transform(model)
+        return m
+
+class DirSerializer(Serializer):
+
+    FORMAT: ModelFormat = ModelFormat.NON_SPECIFIED
+
+    def transform(self, model: Model):
+        dest = self.get_param('dest')
+
+        if Path(dest).exists() and Path(dest).is_dir():
+            raise Exception(f'dest `{dest}` should be a file path.')
+        elif not Path(dest).exists:
+            if dest[-1] in ['\\', '/']:
+                raise Exception(f"dest `{dest}` should file path, not a dir")
+
+        if model.src.is_dir():
             # Zip the dir if the model is in dir form
             tmp_path = Path(get_tmp_path()[:-1])
             shutil.make_archive(str(tmp_path), 'zip', model.src)
             shutil.move(str(tmp_path) + '.zip', dest)
+        else:
+            raise Exception(f'model source `{model.src}` should be a path to a dir.')
 
         return Model(dest)
 
@@ -88,54 +110,38 @@ class H5(FileSerializer):
 
     FORMAT = ModelFormat.H5
 
-    def transform(self, model: Model):
-        return super().transform(model)
-
-    def serialize(self, model: Model, dest: Path) -> Model:
-        return super().serialize(model, dest)
 
 class ONNX(FileSerializer):
 
     FORMAT = ModelFormat.ONNX
 
-    def transform(self, model: Model):
-        return super().transform(model)
-
-    def serialize(self, model: Model, dest: Path) -> Model:
-        return super().serialize(model, dest)
-
 
 class PTH(FileSerializer):
 
     FORMAT = ModelFormat.PTH
 
-    def transform(self, model: Model):
-        return super().transform(model)
-
-    def serialize(self, model: Model, dest: Path) -> Model:
-        return super().serialize(model, dest)
-
 
 class PB(FileSerializer):
 
     FORMAT = ModelFormat.PB
 
-    def transform(self, model: Model):
-        return super().transform(model)
+class TorchTraced(FileSerializer):
 
-    def serialize(self, model: Model, dest: Path) -> Model:
-        return super().serialize(model, dest)
+    FORMAT = ModelFormat.TORCH_TRACED
 
+class TFLITE(FileSerializer):
 
-class SavedModel(FileSerializer):
+    FORMAT = ModelFormat.TFLITE
+
+class SavedModel(DirSerializer):
 
     FORMAT = ModelFormat.SAVED_MODEL
 
-    def transform(self, model: Model):
-        return super().transform(model)
 
-    def serialize(self, model: Model, dest: Path) -> Model:
-        return super().serialize(model, dest)
+class ZippedSavedModel(FileSerializer):
+
+    FORMAT = ModelFormat.ZIPPED_SAVED_MODEL
+
 
 
 class TFKerasModel(Serializer):
@@ -168,6 +174,27 @@ class KerasModel(Serializer):
         return Model(dest)
 
 
+class TF_Session(Serializer):
+    '''
+    Keras 2.5.0 Serializer
+    '''
+
+    FORMAT = ModelFormat.TF_SESSION
+
+    def transform(self, model: Model):
+        import tensorflow
+
+        dest = self.get_param('dest')
+
+        saver = tensorflow.train.Saver()
+
+        sess = model.src
+        sess.run(tensorflow.initialize_all_variables())
+
+        model.src.save(dest.with_suffix('.h5'))
+        shutil.move(dest.with_suffix('.h5'), dest)
+        return Model(dest)
+
 class PytorchModel(Serializer):
     """Use python MRO to check if it contains specific str"""
 
@@ -195,10 +222,23 @@ class PytorchModel(Serializer):
                 "Pytorch model has to have at least one input tensor")
 
         for input_tensor in model.inputs:
-            dummy_inputs.append(torch.rand(*input_tensor.shape))
+            shape = list(input_tensor.shape)
+            shape[0] = 1
+            shape = tuple(shape)
+            dummy_inputs.append(torch.rand(*shape))
 
-        torch.onnx.export(model.src, tuple(dummy_inputs), dest,
-                          input_names=[x.name for x in model.inputs],
-                          output_names=[x.name for x in model.outputs])
+        # torch.onnx.export(model.src, tuple(dummy_inputs), dest,
+        #                   input_names=[x.name for x in model.inputs],
+        #                   output_names=[x.name for x in model.outputs])
+
+        try:
+            traced_module = torch.jit.trace(model.src, tuple(dummy_inputs))
+        except ValueError as ve:
+            logger.error("Unable to save the Pytorch model. "
+                         "Specify `inputs` in project.add_model() "
+                         "may solve this issue.")
+            raise ve
+
+        torch.jit.save(traced_module, dest)
 
         return Model(dest)
